@@ -5,104 +5,91 @@
   
   # resolving the `time' argument
   # --------------------------------------------------------------------
-  
-  NT <- NROW(time)
-  
-  if (is.list(time) && NT==2){
+  if (is.matrix(time)) time <- data.frame(time)
+  if (class(time)=="list"){
+    if (length(time) !=2 || length(time[[1]])!=length(time[[2]]))
+      stop("Argument time has a wrong format")
+    time <- data.frame(time)
+  }
+  if (is.data.frame(time)){
     cens.type <- "intervalCensored"
     L <- time[[1]]
     R <- time[[2]]
+    N <- length(L)
     stopifnot(is.numeric(L))
     stopifnot(is.numeric(R))
-  }
-  else{
-    if (NCOL(time)==2){
-      cens.type <- "intervalCensored"
-      L <- time[,1,drop=TRUE]
-      R <- time[,2,drop=TRUE]
-      stopifnot(is.numeric(L))
-      stopifnot(is.numeric(R))
-    }
-
-    else{
-      N <- NT
-      stopifnot(is.numeric(time))
-      cens.type <- "rightCensored"
-      status <- rep(1,NT) ## only temporary
-    }
-  }
-  
-  if (cens.type=="intervalCensored"){
-    N <- length(L)
-    stopifnot(length(L)==length(R))
     stopifnot(all(L<=R || is.na(R)))
     status <- rep(2,N)
     status[L==R] <- 1
-    status[is.infinite(R) | is.na(R) | as.character(R)==cens.code] <- 0
+    status[is.infinite(R) | is.na(R) | (L!=R & as.character(R)==cens.code)] <- 0
+    ## the last part of the condition achieves to things:
+    ##     1. for multi-state models allow transitions to a censored state
+    ##     2. to ignore this, if an event occured exactly at time 0 and 0 is the cens.code
     R[status==0] <- Inf
   }
+  else{
+    stopifnot(is.numeric(time))
+    cens.type <- "rightCensored"
+    N <- length(time)
+    status <- rep(1,N) ## temporary dummy
+  }
   
-  # resolving argument `event'
+  # resolving the argument `event' 
   # --------------------------------------------------------------------
 
   if (missing(event)){
     model <- "onejump"
-    if (cens.type=="intervalCensored"){
-      event <-  rep(1,N)
-    }
-    else {
-      event <-  rep(1,N)
-      warning("argument `event' is missing:\nassume uncensored observations of a survival model\nwith only one event per subject")
-    }
+    event <-  rep(1,N)
+    warning("Argument event is missing:\nassume observations of a survival model\nand only one event per subject")
   }
   else{
-    NE <- NROW(event)
-    if (!is.null(id)){
-      if (class(event) %in% c("list","data.frame"))
-        stop("When id is given event must be a vector\nshowing the transitions of a multi states model in longitudinal form.")
-      model <- "multi.states"
-      if (cens.type=="intervalCensored"){
-        sorted <- order(id,L)
-        L <- L[sorted]
-        R <- R[sorted]
-        L <- L[duplicated(id)] ## remove the resp. first time
-        R <- R[duplicated(id)] ## remove the resp. first time 
-        status <- status[sorted]
-      }
-      else{
-        sorted <- order(id,time)
-        time <- time[sorted]
-        time <- time[duplicated(id)] ## remove the resp. first time
-        status <- status[duplicated(id)]
-        ## status <- status[sorted] consists only of 1's
-      }
-      last.id <- c(diff(id[sorted]) != 0, 1)
-      first.id <- c(1, diff(id[sorted]) != 0)
-      from <- event[sorted][last.id!=1]
-      to <- event[sorted][first.id!=1]
-      status[is.na(to) | is.infinite(to) | as.character(to)==cens.code] <- 0
+    if (is.matrix(event)) event <- data.frame(event)
+    if (class(event)=="list"){
+      if (length(event) !=2 || length(event[[1]])!=length(event[[2]]))
+        stop("Argument event has a wrong format")
+      event <- data.frame(event)
     }
-    else{
-      if (is.list(event) && NE==2){
-        model <- "multi.states"
-        from <- event[[1]]
-        to <- event[[2]]
-        status[is.na(to) | is.infinite(to) | as.character(to)==cens.code] <- 0
+    if (!is.data.frame(event)){
+      if (is.null(id)){
+        model <- "onejump"
+        if (is.logical(event)) event <- as.numeric(event)
+        status[is.na(event) | is.infinite(event) | as.character(event)==cens.code] <- 0
       }
       else{
-        if (NCOL(event)==2){
-          model <- "multi.states"
-          from <- event[,1,drop=TRUE]
-          to <- event[,2,drop=TRUE]
-          status[is.na(to) | is.infinite(to) | as.character(to)==cens.code] <- 0
+        ## inFormat <- "longitudinal"
+        stopifnot(is.numeric(id) || is.factor(id))
+        model <- "multi.states"
+        if (cens.type=="intervalCensored"){
+          stop("Dont know the order of transitions for interval censored observations.")
         }
         else{
-          if (is.logical(event)) event <- as.numeric(event)
-          status[is.na(event) | is.infinite(event) | as.character(event)==cens.code] <- 0
-          model <- "onejump"
+          # 1. sort the observations by id and time
+          sorted <- order(id,time)
+          time <- time[sorted]
+          ## status <- status[sorted] consists only of 1's
+          id <- id[sorted]
+          event <- event[sorted]
+          # time <- time[duplicated(id)] ## remove the resp. first time
+          # status <- status[duplicated(id)]
+          last.id <- c(diff(id[sorted]) != 0, 1)
+          first.id <- c(1, diff(id[sorted]) != 0)
+          from <- event[last.id!=1]
+          to <- event[first.id!=1]
+          # 2. get back to the original order
+          time <- time[sorted]
+          id <- id[sorted]
+          event <- event[sorted]          
+          status[is.na(to) | is.infinite(to) | as.character(to)==cens.code] <- 0
         }
       }
     }
+    else{
+      ## inFormat <- "from2to"
+      model <- "multi.states"
+      from <- event[[1]]
+      to <- event[[2]]
+      status[is.na(to) | is.infinite(to) | as.character(to)==cens.code] <- 0
+    }
   }
   
   if (all(status==0)) stop("All observations are censored")
@@ -114,6 +101,7 @@
     # --------------------------------------------------------------------
 
     if (is.factor(event)){
+      event <- factor(event) # drop unused levels
       states <- levels(event)
       ## states <- states[match(state.order,states)]
     }
@@ -157,7 +145,7 @@
     
     eventISfactor <- as.numeric(is.factor(from)) + as.numeric(is.factor(to))
 
-    if (eventISfactor==1) stop("Components of `event' have different classes")
+    if (eventISfactor==1) stop("Components of event have different classes")
     
     if (eventISfactor==2)
       states <- unique(c(levels(from),levels(to)))
@@ -167,9 +155,9 @@
     states <- as.character(states[states!=cens.code])
     ## states <- states[match(state.order,states)]
     
-    if (cens.code %in% levels(from)) stop("Code for censored data used wrongly in argument `event'")
+    if (cens.code %in% levels(from)) stop("Code for censored data used wrongly in argument event")
 
-    if (cens.code %in% levels(from)) stop("Code for censored data used wrongly in argument `event'")
+    if (cens.code %in% levels(from)) stop("Code for censored data used wrongly in argument event")
     
     if (cens.type=="intervalCensored"){
       history <- cbind(L=L,
@@ -186,6 +174,7 @@
                        to=as.integer(factor(to,levels=c(cens.code,states))))
     }
   }
+  if (!is.null(id)) history <- cbind(history,id)
   class(history) <- "Hist"
   attr(history,"states") <- states
   attr(history,"cens.type") <- cens.type
@@ -226,3 +215,14 @@ str.Hist <- function(x){
   str(x)
 }
 
+states <- function(x){
+  UseMethod("states")
+}
+
+states.Hist <- function(x){
+  attr(x,"states")
+}
+
+states.prodlim <- function(x){
+  attr(x$model.response,"states")
+}
