@@ -8,7 +8,16 @@ predictSurvIndividual <- function(object,
   else{
     N <- length(obs.times)
     findex <- row.match(object$model.matrix,object$X)
-    psurv <- .C("predict_individual_survival",pred=double(N),as.double(object$surv),as.double(object$time),as.double(obs.times),as.integer(object$first.strata[findex]),as.integer(object$size.strata[findex]),as.integer(N),as.integer(lag),NAOK=FALSE,PACKAGE="prodlim")$pred
-  }
+    psurv <- .C("predict_individual_survival",
+                pred=double(N),
+                as.double(object$surv),
+                as.double(object$time),
+                as.double(obs.times),
+                as.integer(object$first.strata[findex]),
+                as.integer(object$size.strata[findex]),
+                as.integer(N),
+                as.integer(lag),
+                NAOK=FALSE,
+                PACKAGE="prodlim")$pred}
   psurv
 }
